/* Tutorial on Reverse Search */
/* Section 4: C Implementation for Permutations */
/* Solution to Ex 2.2 topological sorts */

/* Input: enter number n of vertices and m edges */
/*        followed by m pairs i j with i < j     */ 

#include <stdio.h>
#define TRUE 1
#define FALSE 0

typedef int perm[100];

int n;				/* global variable for size of permutation */
int A[100][100];                /* holds acyclic digraph compatible with 1,2,...,n */

void output (perm v)
{
  int i;
  for (i = 1; i <= n; i++)
    printf (" %d", v[i]);
  printf ("\n");
}

void copy (perm w, perm v)
{
  int i;

  for (i = 1; i <= n; i++)
    w[i] = v[i];
}

int equal (perm w, perm v)
{
  int i;
  for (i = 1; i <= n; i++)
    if (w[i] != v[i])
      return FALSE;
  return TRUE;
}

int root (perm v)
{
  int i;
  for (i = 1; i <= n; i++)
    if (v[i] != i)
      return FALSE;
  return TRUE;
}

void swap (perm v, int j)
{
  int t;
  t = v[j];
  v[j] = v[j + 1];
  v[j + 1] = t;
}

void Adj (perm v, int j)	/* adjacency oracle */
{
  swap (v, j);
}

void f (perm v)			/* local search function */
{
  int i=1;
  while ( i < n && v[i] <= v[i + 1] ) i++;
  if (i < n)
    swap (v, i);
}

int reverse (perm v, int j)
{
  if(A[v[j]][v[j+1]]==1) return (FALSE);  /* v[j]v[j+1] is an edge */
  perm w;
  copy (w, v);
  Adj (w, j);
  f (w);
  return equal (v, w);
}

int backtrack (perm v)
{
  perm child;
  int j = 0;
  int found = FALSE;
  copy (child, v);
  f (v);			/* v is parent of child */
  do{
      j++;
      Adj (v, j);               /* v replaced by jth child */
      found = equal (child, v);
      swap (v, j);		/* restore v */
     }  while (!found);
  return j;
}

int reversesearch (perm v, int maxdeg)
{
  int j=0, count=1;
  output (v);
  while ( j <= maxdeg )
    {
      do j++;
          while (j <= maxdeg && !reverse (v, j));
      if (j <= maxdeg)
        {
          Adj (v, j);
          output (v); count++;
          j = 0;
        }
      else if(! root(v))
          j = backtrack (v);
    }
  return count;
}

int main ()				/* simple reverse search for permutations */
{
  int v[100];
  int i,j,k, count;
  int m;                               /* number of graph edges */

  for (i=1; i<=n; i++)
     for (j=1; j<=n; j++)
       A[i][j]=0;
  printf ("\nEnter number of vertices and edges:");
  scanf ("%d %d", &n,&m);
  printf ("\nEnter %d edge(s) as pairs i j, i<j : \n", m);
  for (k=1; k<=m; k++)
    {
     scanf("%d %d",&i,&j);
     if (i >= j)
       {  printf ("\nEdge must have form i<j");
          return FALSE;
       }
     A[i][j]=1;
    }

  for (i = 1; i <= n; i++)
    v[i] = i;

  count = reversesearch (v, n - 1);
  printf ("\nnumber of permutations=%d\n", count);
  return 0;
}
